#!/usr/bin/perl

# モジュール読み込み

use Time::Local;
use Device::SerialPort;
use Time::HiRes 'sleep';

# 定義

# シリアルポートは引数から指定する
$SERIAL_dev = $ARGV[0];
# ただし指定しなかった場合はttyACM0とする
if($SERIAL_dev eq '') {
  $SERIAL_dev = "/dev/ttyACM0";
}
# ポートオープン時の待ち時間
$SERIAL_quiet = 0.7; # sec
# 出力ログ
$SERIAL_log = "/var/log/aviosys_8870_ACM0";
# 読み込みインターバル
$SERIAL_int = 0.05; # sec
# 読み込みバッファサイズ
$SERIAL_bytes = 200; # char
# 標準出力などはブロックバッファ
$| = 1;

# ポートオープン
my $objtty;
$objtty = new Device::SerialPort($SERIAL_dev, $SERIAL_quiet)
  or die "ERROR: Cannot open device $SERIAL_dev $!\n";

# ポート設定
$objtty->baudrate(19200);
$objtty->databits(8);
#$dbjtty->parity("none"); # 何故か対応していない言われたので。。。
$objtty->stopbits(1);
$objtty->handshake("none");
$objtty->write_settings()
  or die "ERROR: Cannot setup device $SERIAL_dev $!\n";

# 送受信用変数
my $SERIAL_rx_str, $SERIAL_rx_bytes;
my $SERIAL_tx_str, $SERIAL_tx_bytes;

# 定期出力OFF
$SERIAL_tx_bytes = $objtty->write("auto off\n");

#print time . ",102,$SERIAL_dev,clearing buffer...\n";
# 溜まっている受信バッファをすべて流す
($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
  or die "ERROR: Cannot read device $SERIAL_dev $!\n";
do {
  ($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
    or die "ERROR: Cannot read device $SERIAL_dev $!\n";
} until($SERIAL_rx_bytes == 0);

my $time_start = time;
my $time_count = 0;
my $time_error = 0;
my $amp_max = 0, $amp_min = 0, $amp_avg = 0;

# 延々受信ループ
while(1) {
  
  # 待ち
  sleep($SERIAL_int);
  
  # 秒数が変わったので出力
  if($time_start < time) {
    if($time_count > 0) {
      $amp_avg /= $time_count;
    }
    print "$time_start,$time_count,";
    printf "%1.1f,%1.1f,%1.2f",$amp_min,$amp_max,$amp_avg;
    print ",$time_error\n";
    # 初期化
    $time_start = time;
    $time_count = 0;
    $time_error = 0;
    $amp_max = 0;
    $amp_min = 0;
    $amp_avg = 0;
  }
  
  # コマンド
  $SERIAL_tx_bytes = $objtty->write("read\n");
  
  ($SERIAL_rx_bytes, $SERIAL_rx_str) = $objtty->read($SERIAL_bytes)
#    or print "ERROR: Cannot read device $SERIAL_dev $!\n";
    or $time_error++;
  
  if($SERIAL_rx_bytes == 0) {
    $time_error++;
    next;
  }
  
  chomp $SERIAL_rx_str;
  foreach(split(/\n/,$SERIAL_rx_str)) {
  if($_ =~ /^(\d\d\.\d)A/) {
    #print time . ",200,$SERIAL_dev,$_\n";
    $time_count++;
    if($amp_max < $1) { $amp_max = $1; }
    if($amp_min > $1 || $amp_min == 0) { $amp_min = $1; }
    $amp_avg += $1;
    #print "$1-";
  } else {
    #print time . ",503,$SERIAL_dev,$_\n";
  }
  }
}

$objtty->close()
  or warn "WARNING: Cannot clone device $SERIAL_dev $!\n";


exit 0;

